#ifndef GGAP_WORKSHEET_P_H
#define GGAP_WORKSHEET_P_H

#include <QStringListModel>
#include <QCompleter>
#include <QTextCursor>
#include "ggap/worksheet.h"
#include "moows/wstextdocument.h"
#include "moows/wstextblock.h"

namespace ggap {

class Worksheet;

class WsCompleter : public QCompleter {
    Q_OBJECT

    Worksheet *ws;
    QStringListModel model;
    QStringList items;
    bool model_set;
    QTextCursor completedRegion;

private:
    void ensureModelSet();
    void unsetModel();
    bool eventFilter(QObject *obj, QEvent *event);

private Q_SLOTS:
    void doActivated(const QString &text);
    void doUnsetModel();

public:
    WsCompleter(Worksheet *ws);

    void hide();
    bool popupVisible() const;

    void setRegion(const QTextCursor &region);
    QTextCursor region() const;
    QTextCursor regionAtCursor() const;

    void clear();
    void addData(const char *data, uint size);
    void removeData(const char *data, uint size);
};

namespace parser {
class ErrorInfo;
}

class WorksheetPrivate {
    M_DECLARE_PUBLIC(Worksheet)
    friend class WorksheetDocument;

    WorksheetDocument *wmdoc;
    WorksheetScriptProxy *script_proxy;
    GapProcess *proc;
    WsCompleter completer;
    QString file_type;

    int width;
    int height;
    bool resize_queued;
    Worksheet::Mode mode;

    void startGap();
    void startGap(bool newFile, const QString &workspace, const QString &working_dir);
    void forkCommand(const GapCommand &cmd, GapProcess::Flags flags);
    void killGap();
    void queueResize();
    void writeInput(const QStringList &lines, bool askCompletions = false);
    void writeFakeInput(const QStringList &lines, const QString &history, bool askCompletions = false);
    void writeErrors(const QList<parser::ErrorInfo> &parser);

    bool saveWorkspace(const QString &filename, QString &error);

    WorksheetPrivate(Worksheet *ws);
    ~WorksheetPrivate();
};

class WorksheetDocument : public moo::Document {
    Worksheet *ws;

    bool loadFile(const QUrl &url, QString &error)
    {
        return ws->loadFile(url, error);
    }

    bool saveFile(const QUrl &url, const QString &type, QString &error)
    {
        return ws->saveFile(url, type, error);
    }

    bool isModified() const
    {
        return ws->document().isModified();
    }

    void setModified(bool modified)
    {
        ws->document().setModified(modified);
    }

    bool isEmpty() const
    {
        if (isModified() || !isUntitled())
            return false;

        const moo::ws::PromptBlock *block;
        block = dynamic_cast<const moo::ws::PromptBlock*>(ws->firstBlock());
        return block && !block->next() && block->text().isEmpty();
    }

    QString documentType() const
    {
        return ws->impl->file_type;
    }

public:
    WorksheetDocument(Worksheet *ws) :
	moo::Document(ws),
        ws(ws)
    {
    }
};

}

#endif // GGAP_WORKSHEET_P_H
